@@ -9,60 +9,51 @@
 
 #include "common.h"
 
+typedef struct
+{
+	GladeXML *xml;
+	GtkWidget *notebook;
+	GtkWidget *lb_userwin;
+	GtkWidget *lb_userwin_online;
+	GtkWidget *lb_userwin_offline;
+	GtkWidget *lb_userwin_notinlist;
+	GtkWidget *head_online;
+	GtkWidget *head_offline;
+	GtkWidget *head_notinlist;
+	GtkWidget *not_contents;
+	GtkWidget *window;
+} _MainData;
+
+extern _MainData *MainData;
+
 extern GSList *Contacts;
-extern Contact_Member New_Contact;
-extern WORD seq_num;
+
 extern DWORD our_ip;
 extern DWORD our_port;
-extern gboolean Quit;
 extern DWORD Current_Status;
-extern DWORD last_recv_uin;
 extern gchar *passwd;
 extern gchar *server;
 extern DWORD remote_port;
-extern DWORD set_status;
 extern gchar *Away_Message;
 extern gboolean Done_Login;
-extern gint connecting;
 
 extern _programs *programs;
 
 extern gboolean search_in_progress;
-extern GtkWidget *log_list;
-extern GtkWidget *log_window;
-
-extern gint autohide_time;
-extern gint max_tcp_port;
-extern gint min_tcp_port;
-
-extern gboolean sound_disable;
 
 extern BYTE chat_fg_red, chat_fg_green, chat_fg_blue;
 extern BYTE chat_bg_red, chat_bg_green, chat_bg_blue;
 
-extern GtkWidget *statusbar;
-extern guint sound_pid;
-
-extern GtkWidget *progress_indicator;
-extern GtkTooltips *status_tooltip;
-
 extern gboolean is_new_user;
 
 extern USER_INFO_STRUCT our_user_info;
 
-extern gchar *WindowTitle;
 extern gint WindowWidth, WindowHeight;
+extern gint WindowX, WindowY;
 
 extern gchar *configfilename;
 
 extern gint tcp_gdk_input;
-extern gint udp_gdk_input;
-
-extern guint next_resend;
-
-extern struct sokandlb *MainData;
-
-extern gint Connected;
 
 extern GdkFont *ChatFont;
 extern gchar *ChatFontString;
@@ -73,8 +64,6 @@ extern USER_INFO_PTR our_info;
 
 extern GtkWidget *app;
 
-extern gboolean been_thru_init;
-
 extern gboolean enable_online_events;
 
 extern gboolean webpresence;
@@ -84,12 +73,22 @@ extern _toggles *toggles;
 extern GtkWidget *eyes_animated;
 extern GtkWidget *eyes_still;
 
+/* for server side contacts list */
+extern gboolean srvlist_exist;   /* server side list exist for this user? */
+extern guint32 list_time_stamp;  /* time stamp of the list */
+extern guint16 record_cnt;       /* record count */
+extern guint16 status_uid;       /* for change status on server list */
+extern gboolean sane_cl;         /* is the contact list sane? or checked? */
+
+extern GSList *Groups;
+
 void create_tcp_line( void );
 void ready_set( void );
-void create_animator( GtkWidget *hbox_parent );
-void toggle_log_window( GtkWidget *widget, gpointer data );
+void create_animator (void);
 void init( void );
+void about( GtkWidget *widget, gpointer data );
 void go_url_home (void);
 void go_url_updates (void);
+void icq_quit( GtkWidget *widget, gpointer data );
 
 #endif /* __GNOMEICU_H__ */
