@@ -4,70 +4,53 @@
  GnomeICU
 *********************************/
 
-/*** GnomeICU header files ***/
 #include "common.h"
+#include "applet.h"
 #include "dirbrowser.h"
+#include "gnomeicu.h"
+#include "gtkfunc.h"
+#include "history.h"
+#include "icons.h"
+#include "kanji_conv.h"
+#include "packetprint.h"
+#include "response.h"
+#include "sendcontact.h"
+#include "showlist.h"
+#include "tcp.h"
+#include "util.h"
+#include "v7send.h"
 
-/*** Toplevel header files ***/
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/time.h>
-#include <fcntl.h>
-#include <time.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <assert.h>
-#include <sys/wait.h>
-#include <signal.h>
-#ifdef HAVE_SOCKS5
-#define SOCKS
-#include <socks.h>
-#endif
+gint min_tcp_port = 4000;
+gint max_tcp_port = 5000;
 
 GtkWidget *filesel;
 
-typedef struct
-{
-	struct sokandlb *data;
-	int i;
-} dataandint;
-
 /* 127.0.0.1 is localhost */
 const DWORD LOCALHOST = 0x7F000001;
 
 /*** Local function declarations ***/
 static int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data );
-static int TCPAckPacket( int sock, GSList *contact, WORD cmd, int seq );
+static int TCPAckPacket( int sock, WORD cmd, int seq );
 static void TCPProcessPacket( BYTE *packet, int packet_length, int sock );
 
-static int TCPConnectFile( struct xfer_struct *xfer );
-static int TCPFileHandshake( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer );
-static int TCPFileReadServer( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer );
-static int TCPFileReadClient( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer );
-static void TCPSendFileInfo( struct xfer_struct *xfer );
-static int TCPSendPartFile( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer );
+static int TCPConnectFile( XferInfo *xfer );
+static int TCPFileHandshake( GIOChannel *source, GIOCondition condition, XferInfo *xfer );
+static int TCPFileReadServer( GIOChannel *source, GIOCondition condition, XferInfo *xfer );
+static int TCPFileReadClient( GIOChannel *source, GIOCondition condition, XferInfo *xfer );
+static void TCPSendFileInfo( XferInfo *xfer );
+static int TCPSendPartFile( GIOChannel *source, GIOCondition condition, XferInfo *xfer );
 
-static void file_place( struct xfer_struct *xfer );
-static void set_dir(gchar *dir, struct xfer_struct *xfer);
+static void file_place( XferInfo *xfer );
+static void set_dir(gchar *dir, XferInfo *xfer);
 static void save_incoming_file( GtkWidget *widget, gpointer data );
 
-static int next_file( struct xfer_struct *xfer );
+static int next_file( XferInfo *xfer );
 
 static int TCPFinishConnection( GIOChannel *source, GIOCondition cond, gpointer data );
 static int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data );
 static int TCPTimeout( tcp_message *m );
 
-static void timeout_continue( GtkWidget *widget, tcp_message *m );
-static void timeout_server( GtkWidget *widget, tcp_message *m );
-static void timeout_cancel( GtkWidget *widget, tcp_message *m );
+static void timeout_clicked( GnomeDialog *dialog, gint button, tcp_message *m );
 
 GSList *pending_xfers = 0;
 
@@ -290,9 +273,11 @@ int TCPSendMessage( UIN_T uin, gchar *msg )
 	packet.body = msg;
 
 	contact = Find_User( uin );
-	if( contact == NULL || kontakt->force_override )
+	if( contact == NULL || kontakt->force_override ||
+            !kontakt->has_direct_connect)
 		return FALSE;
 
+
 	DW_2_IP( packet.tail.ip, our_ip );
 	DW_2_IP( packet.tail.real_ip, our_ip );
 	DW_2_Chars( packet.tail.port, our_port );
@@ -385,6 +370,7 @@ int TCPSendURL( UIN_T uin, gchar *msg )
 	g_print( "TCPSendURL\n" );
 #endif
 
+
 	DW_2_Chars( packet.head.uin_a, our_info->uin );
 	Word_2_Chars( packet.head.version, 0x0003 );
 	Word_2_Chars( packet.head.cmd, ICQ_CMDxTCP_START );
@@ -404,6 +390,8 @@ int TCPSendURL( UIN_T uin, gchar *msg )
 	contact = Find_User( uin );
 	if( contact == NULL )
 		return 0;
+        if (!kontakt->has_direct_connect)
+          return 0;
 
 	DW_2_Chars( packet.tail.seq, kontakt->tcp_seq );
 
@@ -507,6 +495,9 @@ int TCPRetrieveAwayMessage( GSList *contact, gpointer data )
 	g_print( "TCPRetrieveAwayMessage\n" );
 #endif
 
+        if (!kontakt->has_direct_connect)
+          return FALSE;
+
 	switch( kontakt->status & 0xffff )
 	{
 		case STATUS_AWAY:
@@ -611,6 +602,9 @@ int TCPRetrieveVersion( GSList *contact, gpointer data )
 	g_print( "TCPRetrieveVersion\n" );
 #endif
 
+        if (!kontakt->has_direct_connect)
+          return FALSE;
+
 	DW_2_Chars( packet.head.uin_a, our_info->uin );
 	Word_2_Chars( packet.head.version, 0x0003 );
 	Word_2_Chars( packet.head.cmd, ICQ_CMDxTCP_START );
@@ -645,7 +639,7 @@ int TCPRetrieveVersion( GSList *contact, gpointer data )
 	                         kontakt->tcp_seq--, FALSE );
 
 	sock = TCPGainConnection( kontakt->current_ip, kontakt->port, contact );
-        
+
 	if( sock == -2 ) /* We're waiting for a connection */
 		return TRUE;
 
@@ -657,7 +651,7 @@ int TCPRetrieveVersion( GSList *contact, gpointer data )
 	return 1;
 }
 
-int TCPAcceptFile( struct xfer_struct *xfer )
+int TCPAcceptFile( XferInfo *xfer )
 {
 	BYTE *buffer;
 	unsigned short intsize;
@@ -757,6 +751,10 @@ int TCPAcceptFile( struct xfer_struct *xfer )
 	DW_2_Chars( pack_tail.back_port, backport );
 	DW_2_Chars( pack_tail.seq, xfer->seq );
 
+	if (toggles->auto_accept_xfers) {
+		xfer->auto_save_path = g_strdup(toggles->auto_accept_path);
+	}
+
 	if( xfer->remote_contact->sok != -1 )
 	{
 		intsize = sizeof( tcp_head ) + sizeof( tcp_tail ) + 1;
@@ -779,7 +777,7 @@ int TCPAcceptFile( struct xfer_struct *xfer )
 	return 1;
 }
 
-int TCPRefuseFile( struct xfer_struct *xfer )
+int TCPRefuseFile( XferInfo *xfer )
 {
 	BYTE *buffer;
 	unsigned short intsize;
@@ -805,9 +803,9 @@ int TCPRefuseFile( struct xfer_struct *xfer )
                 BYTE back_port[4];
                 BYTE one[2];
                 BYTE zero;
-                BYTE bigzero[4];  
-                BYTE portb[4];   
-                BYTE seq[4]; 
+                BYTE bigzero[4];
+                BYTE portb[4];
+                BYTE seq[4];
         } tcp_tail;
 
 	tcp_head pack_head;
@@ -833,7 +831,7 @@ int TCPRefuseFile( struct xfer_struct *xfer )
         DW_2_Chars( pack_tail.porta, our_port );
         pack_tail.junk = 0x04;
         DW_2_Chars( pack_tail.status, ICQ_ACKxTCP_REFUSE );
-        DW_2_Chars( pack_tail.portb, 0 );  
+        DW_2_Chars( pack_tail.portb, 0 );
         DW_2_Chars( pack_tail.one, 0x0001 );
         pack_tail.zero = 0x00;
         DW_2_Chars( pack_tail.bigzero, 0 );
@@ -914,7 +912,7 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
 	gchar *nopathfile = 0;
 
 	struct file_struct *fileinfo;
-	struct xfer_struct *xferinfo;
+	XferInfo *xferinfo;
 
 	GSList *file;
 	GSList *contact;
@@ -924,10 +922,10 @@ int TCPSendFileRequest( UIN_T uin, const gchar *msg, GSList *files )
 #endif
 	
 	contact = Find_User( uin );
-	if( contact == NULL )
+	if( contact == NULL || !kontakt->has_direct_connect)
 		return FALSE;
 
-	xferinfo = g_malloc0(sizeof(struct xfer_struct));
+	xferinfo = g_malloc0(sizeof(XferInfo));
 
 	xferinfo->seq = kontakt->tcp_seq;
 	xferinfo->remote_contact = kontakt;
@@ -1129,7 +1127,7 @@ void TCPSendContactList( GtkWidget *widget, gpointer data )
 	packet.body = msg;
 
 	contact = Find_User( uin );
-	if( contact == NULL )
+	if( contact == NULL  || !kontakt->has_direct_connect)
 		return;
 
 	DW_2_IP( packet.tail.ip, our_ip );
@@ -1242,7 +1240,7 @@ int TCPInitChannel( GIOChannel *source, GIOCondition cond, gpointer data )
 	return FALSE;
 }
 
-int TCPAckPacket( int sock, GSList *contact, WORD cmd, int seq )
+int TCPAckPacket( int sock, WORD cmd, int seq )
 {
 	BYTE *buffer;
 	unsigned short intsize;
@@ -1390,18 +1388,21 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 	} tcp_file;
 
 	gchar *message;
-	gchar *away_message;
+	gchar *desc;
 	tcp_head pack_head;
 	tcp_tail pack_tail;
 	tcp_file pack_file;
 	tcp_chat pack_chat;
 
-	GSList *contact;
 	GSList *messages;
 	tcp_message *m = NULL;
+	GSList *contact;
 	ChatContact *ccontact;
 
-	DWORD i;
+        gchar *countstr, *tmp = NULL;
+        ContactPair *cpair;
+        GSList *contacts = NULL;
+        int i, count;
 
 #ifdef TRACE_FUNCTION
 	g_print( "TCPProcessPacket\n" );
@@ -1455,9 +1456,9 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 		if (pack_chat.name_len  > 0) {
 		     pack_chat.name = g_malloc0( pack_chat.name_len );
 		     memcpy( pack_chat.name, size, pack_chat.name_len );
-		} else 
+		} else
 		     size += 1;
-	      
+	
 		size += pack_chat.name_len;
 		pack_chat.revport = Chars_2_Word( size );
 		size += 2;
@@ -1465,7 +1466,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 		pack_chat.port = Chars_2_DW( size );
 		size += 4;
 	}
- 
+
 	if ( pack_head.cmd == ICQ_CMDxTCP_FILE )
 	{
 		pack_file.revport = Chars_2_Word( size );
@@ -1478,8 +1479,8 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 		size += pack_file.name_len;
 		pack_file.size = Chars_2_DW( size );
 		size += 4;
-		pack_file.port = Chars_2_Word( size ); 
-		size += 4; 
+		pack_file.port = Chars_2_Word( size );
+		size += 4;
 	}
 
 	pack_tail.seq = Chars_2_DW (size);
@@ -1492,7 +1493,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 	pack_tail.ip_local = ( ( i << 24 ) | ( ( i & 0xff00 ) << 8 ) | ( ( i & 0xff0000 ) >> 8 ) | ( i >> 24 ) );
 
     if( toggles->no_new_users == TRUE){
-        TCPAckPacket( sock, NULL, pack_head.cmd, pack_tail.seq );
+        TCPAckPacket( sock, pack_head.cmd, pack_tail.seq );
         return;
     }
 
@@ -1504,47 +1505,73 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 				case ICQ_CMDxTCP_MSG:
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "TCP MESSAGE(MSG)" );
-					contact = Do_Msg( time( NULL ), 0, message, pack_head.uin1, MESSAGE_TEXT );
-					log_window_add( _("Received Message (TCP)"), pack_head.uin1 );
-					TCPAckPacket( sock, contact, pack_head.cmd, pack_tail.seq );
+					msg_received( pack_head.uin1, message, time( NULL ) );
+					TCPAckPacket( sock, pack_head.cmd, pack_tail.seq );
 					break;
-				case ICQ_CMDxTCP_VERSION:
-					packet_print( packet, packet_length,
-					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "VERSION REQUEST" );
-
-					contact = Find_User( pack_head.uin1 );
-					if( contact == NULL )
-						break;
-
-					TCPAckPacket( sock, contact, ICQ_CMDxTCP_VERSION, pack_tail.seq );
-					break;
-
 				case ICQ_CMDxTCP_READxAWAYxMSG:
 				case ICQ_CMDxTCP_READxOCCxMSG:
 				case ICQ_CMDxTCP_READxDNDxMSG:
 				case ICQ_CMDxTCP_READxNAxMSG:
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "AWAY MSG REQUEST" );
-					contact = Find_User( pack_head.uin1 );
-					if( contact == NULL )
-						break;
 
-					TCPAckPacket( sock, contact, ICQ_CMDxTCP_READxAWAYxMSG,
+					/* FIXME: Do nothing? */
+					TCPAckPacket( sock, ICQ_CMDxTCP_READxAWAYxMSG,
 					              pack_tail.seq );
 					break;
 
 				case ICQ_CMDxTCP_URL:  /* url sent */
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "TCP MESSAGE(URL)" );
-					contact = Do_Msg( time( NULL ), 0, message, pack_head.uin1, MESSAGE_URL );
-					TCPAckPacket( sock, contact, pack_head.cmd, pack_tail.seq );
+ 		     			
+					desc = strchr( message, '\xFE' ) + 1;
+					*(desc-1) = '\0';
+
+					url_received( pack_head.uin1, message, desc, time( NULL ) );
+					TCPAckPacket( sock, pack_head.cmd, pack_tail.seq );
 					break;
 
 				case ICQ_CMDxTCP_CONT_LIST:
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "TCP MESSAGE(CONT_LIST)" );
-					contact = Do_Msg( time( NULL ), 0, message, pack_head.uin1, MESSAGE_CONT_LIST );
-					TCPAckPacket( sock, contact, pack_head.cmd, pack_tail.seq );
+                                        countstr = message;
+                                        tmp = strchr( tmp, '\xFE' );
+                                        if ( tmp == NULL )
+                                          return; /* Bad Packet */
+                                        *tmp = '\0';
+                                        tmp++;
+
+                                        count = atoi(countstr);
+
+                                        if (count == 0)
+                                          return;
+
+                                        for (i=0; i < count; i++) {
+                                          cpair = g_malloc(sizeof(ContactPair));
+
+                                          cpair->textuin = tmp;
+                                          tmp = strchr( tmp, '\xFE' );
+                                          if ( tmp == NULL )
+                                            return; /* Bad Packet */
+                                          *tmp = '\0';
+                                          cpair->textuin = g_strdup(cpair->textuin);
+                                          tmp++;
+
+                                          cpair->nick = tmp;
+                                          tmp = strchr( tmp, '\xFE' );
+                                          if ( tmp == NULL )
+                                            return; /* Bad Packet */
+                                          *tmp = '\0';
+                                          cpair->nick = g_strdup(cpair->nick);
+                                          tmp++;
+
+                                          contacts = g_slist_append(contacts, cpair);
+                                        }
+
+                                        contact_list_received (pack_head.uin1, contacts, time(NULL));
+
+
+					TCPAckPacket( sock, pack_head.cmd, pack_tail.seq );
 					break;
 
 				case ICQ_CMDxTCP_CHAT:
@@ -1554,13 +1581,13 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 				     if( Current_Status == STATUS_FREE_CHAT )
 					  TCPAcceptChat( sock, ccontact);
 				     break;
-				     
+				
 				case ICQ_CMDxTCP_FILE:
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "FILE REQUEST" );
 			
-					contact = Do_File( message, pack_head.uin1, pack_tail.seq, pack_file.name, pack_file.size );
-					break;      
+					Do_File( message, pack_head.uin1, pack_tail.seq, pack_file.name, pack_file.size );
+					break;
 
 				default:
 					break;
@@ -1570,21 +1597,6 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 		case ICQ_CMDxTCP_ACK:
 			switch ( pack_head.cmd )
 			{
-				case ICQ_CMDxTCP_VERSION:
-					contact = Find_User( pack_head.uin1 );
-					if( contact == NULL )
-						break;
-
-					packet_print( packet, packet_length,
-					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "RESPONSE VERSION" );
-
-					away_message = g_strdup_printf( "%s is running\nGnomeICU version:",
-					                                kontakt->nick );
-
-					AMessage_Box( away_message, message, kontakt->uin );
-					g_free( away_message );
-
-					break;
 				case ICQ_CMDxTCP_MSG:
 				case ICQ_CMDxTCP_CONT_LIST:
 			      	case ICQ_CMDxTCP_READxAWAYxMSG:
@@ -1632,26 +1644,28 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 					    (pack_tail.status & 0xffff) == ICQ_ACKxTCP_DND  ||
 					    (pack_tail.status & 0xffff) == ICQ_ACKxTCP_OCC )
 					{
+                                          DWORD status;
 						switch( pack_tail.status )
 						{
-							case ICQ_ACKxTCP_AWAY:
-								away_message = g_strdup_printf( _("User %s is Away:"), kontakt->nick );
-								break;
-							case ICQ_ACKxTCP_NA:
-								away_message = g_strdup_printf( _("User %s is Not Available:"), kontakt->nick );
-								break;
-							case ICQ_ACKxTCP_DND:
-								away_message = g_strdup_printf( _("User %s cannot be disturbed:"), kontakt->nick );
-								break;
-							case ICQ_ACKxTCP_OCC:
-								away_message = g_strdup_printf( _("User %s is Occupied:"), kontakt->nick );
-								break;
-							default:
-								away_message = g_strdup( _("User is in an unknown mode:") );
+                                                case ICQ_ACKxTCP_AWAY:
+                                                  status = STATUS_AWAY;
+                                                  break;
+                                                case ICQ_ACKxTCP_NA:
+                                                  status = STATUS_NA;
+                                                  break;
+                                                case ICQ_ACKxTCP_DND:
+                                                  status = STATUS_DND;
+                                                  break;
+                                                case ICQ_ACKxTCP_OCC:
+                                                  status = STATUS_OCCUPIED;
+                                                  break;
+                                                default:
+                                                  status = STATUS_OFFLINE;
+                                                  break;
 						}
 	
-						AMessage_Box( away_message, message, kontakt->uin );
-						g_free( away_message );
+						recv_awaymsg( kontakt->uin,
+                                                              status, message );
 					}
 					break;
 
@@ -1667,13 +1681,13 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "ACK FILE REQUEST" );
 					
 					for(xfer = pending_xfers;xfer;xfer = xfer->next)
-						if ( ( ((struct xfer_struct*)xfer->data)->remote_contact->uin == pack_head.uin1) && 
-							( ((struct xfer_struct*)xfer->data)->seq == pack_tail.seq) &&
-							( ((struct xfer_struct*)xfer->data)->direction == XFER_DIRECTION_SENDING)) 
+						if ( ( ((XferInfo*)xfer->data)->remote_contact->uin == pack_head.uin1) &&
+							( ((XferInfo*)xfer->data)->seq == pack_tail.seq) &&
+							( ((XferInfo*)xfer->data)->direction == XFER_DIRECTION_SENDING))
 							break;
 
 					if (xfer) {
-						struct xfer_struct *transfer = xfer->data;
+						XferInfo *transfer = xfer->data;
 
 						pending_xfers = g_slist_remove(pending_xfers, transfer);
 
@@ -1691,7 +1705,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 					break;
 			}
 			break;
-      
+
 		case ICQ_CMDxTCP_CANCEL:
 			switch ( pack_head.cmd )
 			{
@@ -1719,7 +1733,7 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 					else
 					{
 					     GSList *temp_storedmsg;
-					     
+					
 					     temp_storedmsg = kontakt->stored_messages;
 					     while (temp_storedmsg != NULL)
 					     {
@@ -1730,21 +1744,21 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 
 					     if (temp_storedmsg != NULL)
 					     {
-						  
+						
 						  g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->message);
 						  g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->chatsessionname);
-						  g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->chatcontact);
+						  g_free(((STORED_MESSAGE_PTR) temp_storedmsg->data)->data);
 						  g_free(temp_storedmsg->data);
 						  kontakt->stored_messages = g_slist_remove( kontakt->stored_messages, temp_storedmsg->data);
-						  
-						  applet_update( Current_Status, NULL );
+						
+						  applet_update( Current_Status, FALSE );
 						  if( g_slist_length( kontakt->stored_messages ) == 0 )
 						  {
-						       kontakt->icon_p = GetIcon_p( kontakt->status );
-						       kontakt->icon_b = GetIcon_b( kontakt->status );
+						       kontakt->icon_p = get_pixmap_for_status( kontakt->status );
+						       kontakt->icon_b = get_bitmap_for_status( kontakt->status );
 						       kontakt->need_update = 1;
 						  }
-						  
+						
 						  Show_Quick_Status();
 					     }
 					}
@@ -1753,35 +1767,35 @@ void TCPProcessPacket( BYTE *packet, int packet_length, int sock )
 					/* This should be reimplemented with a "More Info" button */
 					/* OK_Box( _("Chat Request Refused"), message ); */
 					break;
-      
+
 				case ICQ_CMDxTCP_FILE:
 					packet_print( packet, packet_length,
 					              PACKET_TYPE_TCP | PACKET_DIRECTION_RECEIVE, "FILE REQ CANCEL" );
 
 					for(xfer = pending_xfers;xfer;xfer = xfer->next) {
-						if ( ( ((struct xfer_struct*)xfer->data)->remote_contact->uin == pack_head.uin1) && 
-							( ((struct xfer_struct*)xfer->data)->seq == pack_tail.seq))
+						if ( ( ((XferInfo*)xfer->data)->remote_contact->uin == pack_head.uin1) &&
+							( ((XferInfo*)xfer->data)->seq == pack_tail.seq))
 							break;
 					}
 
-					if (xfer) 
+					if (xfer)
 						cancel_transfer(0, xfer->data);
 
 					break;
-      
+
 				default:
 					break;
 			}
 			break;
-   
+
 		default:
 			break;
 	}
 
 	g_free( message );
-} 
+}
 
-int TCPConnectFile( struct xfer_struct *xfer )
+int TCPConnectFile( XferInfo *xfer )
 {
 	DWORD localport;
 	struct sockaddr_in local, remote;
@@ -1813,7 +1827,8 @@ int TCPConnectFile( struct xfer_struct *xfer )
 	if( xfer->sock_fd > 0 )
 		return xfer->sock_fd;
 
-	if (!xfer->remote_contact->current_ip)
+	if (!xfer->remote_contact->current_ip ||
+            !xfer->remote_contact->has_direct_connect)
 		return -1;
 
 	xfer->sock_fd = socket( PF_INET, SOCK_STREAM, 0 );
@@ -1895,7 +1910,7 @@ int TCPConnectFile( struct xfer_struct *xfer )
 	return xfer->sock_fd;
 }
 
-int TCPFileHandshake( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer )
+int TCPFileHandshake( GIOChannel *source, GIOCondition condition, XferInfo *xfer )
 {
 	int new_sock;
 	int size = sizeof( struct sockaddr );
@@ -1903,7 +1918,7 @@ int TCPFileHandshake( GIOChannel *source, GIOCondition condition, struct xfer_st
 
 #ifdef TRACE_FUNCTION
 	g_print( "TCPFileHandshake\n" );
-#endif	
+#endif
 
 	new_sock = accept( xfer->sock_fd, ( struct sockaddr * )&their_addr, &size );
 
@@ -1925,7 +1940,7 @@ int TCPFileHandshake( GIOChannel *source, GIOCondition condition, struct xfer_st
 	return FALSE;
 }
 
-int TCPFileReadServer(GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer)
+int TCPFileReadServer(GIOChannel *source, GIOCondition condition, XferInfo *xfer)
 {
 	unsigned short packet_size;
 	BYTE *packet;
@@ -1942,7 +1957,7 @@ int TCPFileReadServer(GIOChannel *source, GIOCondition condition, struct xfer_st
 		cancel_transfer(0, xfer);
 		return FALSE;
 	}
-        
+
 	packet_size = GUINT16_FROM_LE (packet_size);
 
 	packet = (BYTE *)g_malloc( packet_size );
@@ -1979,9 +1994,9 @@ int TCPFileReadServer(GIOChannel *source, GIOCondition condition, struct xfer_st
 			xfer, NULL );
 
  		((struct file_struct *)xfer->file_queue->data)->start_time = time( NULL );
- 
+
  		if( xfer->start_time == 0 )
- 			xfer->start_time = time( NULL );  
+ 			xfer->start_time = time( NULL );
 
 		xfer->status = XFER_STATUS_TRANSFERING;
 
@@ -1996,7 +2011,7 @@ int TCPFileReadServer(GIOChannel *source, GIOCondition condition, struct xfer_st
 	return TRUE;
 }
 
-int TCPFileReadClient( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer )
+int TCPFileReadClient( GIOChannel *source, GIOCondition condition, XferInfo *xfer )
 {
 	int file_count, bytes_read, ret = TRUE;
 
@@ -2102,7 +2117,7 @@ int TCPFileReadClient( GIOChannel *source, GIOCondition condition, struct xfer_s
 	return ret;
 }
 
-void TCPSendFileInfo( struct xfer_struct *xfer ) {
+void TCPSendFileInfo( XferInfo *xfer ) {
 	BYTE *buffer;
 	unsigned short packet_size;
 	int index;
@@ -2137,18 +2152,18 @@ void TCPSendFileInfo( struct xfer_struct *xfer ) {
 	g_free( buffer );
 }
 
-int TCPSendPartFile( GIOChannel *source, GIOCondition condition, struct xfer_struct *xfer )
+int TCPSendPartFile( GIOChannel *source, GIOCondition condition, XferInfo *xfer )
 {
 	int bytes;
 	xfer6 pak;	
 	
 #ifdef TRACE_FUNCTION
 	g_print("TCPSendPartFile\n");
-#endif	
+#endif
 
         /* They hung up on us... <sniff> */
-        if ( condition & (G_IO_HUP | G_IO_NVAL | G_IO_ERR) ) 
-                return FALSE;                     
+        if ( condition & (G_IO_HUP | G_IO_NVAL | G_IO_ERR) )
+                return FALSE;
 
 	/* The file we're sending was skipped by someone, commit suicide */
 	if (xfer->status != XFER_STATUS_TRANSFERING)
@@ -2174,7 +2189,7 @@ int TCPSendPartFile( GIOChannel *source, GIOCondition condition, struct xfer_str
 	return TRUE;
 }
 
-void file_place( struct xfer_struct *xfer )
+void file_place( XferInfo *xfer )
 {
 	GtkWidget *filesel;
 #ifdef TRACE_FUNCTION
@@ -2209,14 +2224,14 @@ void file_place( struct xfer_struct *xfer )
 	gtk_widget_show(filesel);
 }
 
-void set_dir(gchar *dir, struct xfer_struct *xfer) {
+void set_dir(gchar *dir, XferInfo *xfer) {
 	xfer->auto_save_path = g_strdup(dir);
 	file_place(xfer);
 }
 
 void save_incoming_file( GtkWidget *widget, gpointer data )
 {
-	struct xfer_struct *xfer = data;
+	XferInfo *xfer = data;
 	struct stat file_stat;
 	xfer3 pak;
 
@@ -2252,15 +2267,15 @@ void save_incoming_file( GtkWidget *widget, gpointer data )
 	}
 
  	((struct file_struct *)xfer->file_queue->data)->start_time = time( NULL );
- 
+
  	if( xfer->start_time == 0 )
  		xfer->start_time = time( NULL );
 
 	return;
 }
 
-void cancel_transfer( GtkWidget *widget, struct xfer_struct *xfer )
-{ 
+void cancel_transfer( GtkWidget *widget, XferInfo *xfer )
+{
 	gchar *msg;
         time_t timedate;
 
@@ -2278,23 +2293,23 @@ void cancel_transfer( GtkWidget *widget, struct xfer_struct *xfer )
 		/* <kludge> */
 		/*  Grrr, or else the last file won't be counted */
 		struct file_struct *completed_file = (struct file_struct *)xfer->file_queue->data;
-        
+
 		xfer->file_queue = g_slist_remove( xfer->file_queue, completed_file );
 		xfer->completed_files = g_slist_append( xfer->completed_files, completed_file );
 		/* </kludge> */
 
                 msg = g_strdup_printf( _("File transfer:\nCompleted transfer of %s from %s."), xfer->filename, xfer->remote_contact->nick );
                 time( &timedate );
-                add_incoming_to_history( xfer->remote_contact->uin, msg, &timedate );
+                history_add_incoming( xfer->remote_contact->uin, msg, &timedate );
                 request_file_action(xfer->completed_files);
                 g_free( msg );
             }
             else
             {
                 msg = g_strdup_printf( _("File transfer:\nCompleted transfer of %s to %s."), xfer->filename, xfer->remote_contact->nick );
-                add_outgoing_to_history( xfer->remote_contact->uin, msg );
+                history_add_outgoing( xfer->remote_contact->uin, msg );
                 g_free( msg );
-                
+
                 msg = g_strdup_printf( _("File transfer completed: %s"), xfer->filename );
                 gnome_ok_dialog( msg );
                 g_free( msg );
@@ -2334,20 +2349,20 @@ void cancel_transfer( GtkWidget *widget, struct xfer_struct *xfer )
 
 	g_slist_free(xfer->file_queue);
 
-        for(file_queue = xfer->completed_files;file_queue;file_queue = file_queue->next) {            
+        for(file_queue = xfer->completed_files;file_queue;file_queue = file_queue->next) {
             g_free( ((struct file_struct*)file_queue->data)->short_filename);
             g_free( ((struct file_struct*)file_queue->data)->full_filename);
             g_free( (struct file_struct*)file_queue->data );
         }
-                
+
         g_slist_free(xfer->completed_files);
 
-	g_free(xfer); 
+	g_free(xfer);
 
 	return;
 }
 
-void skip_file( GtkWidget *widget, struct xfer_struct *xfer) {
+void skip_file( GtkWidget *widget, XferInfo *xfer) {
 	xfer4 pak;
 
 	if (xfer->direction == XFER_DIRECTION_SENDING)
@@ -2360,7 +2375,7 @@ void skip_file( GtkWidget *widget, struct xfer_struct *xfer) {
 	}
 }
 
-int next_file( struct xfer_struct *xfer ) {
+int next_file( XferInfo *xfer ) {
 	struct file_struct *completed_file = (struct file_struct *)xfer->file_queue->data;
 
 	xfer->file_queue = g_slist_remove( xfer->file_queue, completed_file );
@@ -2531,12 +2546,15 @@ int TCPReadPacket( GIOChannel *source, GIOCondition cond, gpointer data )
 
 static int TCPTimeout( tcp_message *m )
 {
-	GtkWidget *label;
-        GString *SendError;
+	gchar *error;
 	static int first = TRUE;
 
 	GSList *contact;
 
+#ifdef TRACE_FUNCTION
+	g_print( "TCPTimeout\n" );
+#endif
+
 	if( first )
 	{
 		first = FALSE;
@@ -2552,39 +2570,36 @@ static int TCPTimeout( tcp_message *m )
 	if( kontakt->force_override == TRUE )
 	{
 		if( m->text != NULL )
-			icq_sendmsg( m->uin, m->text, TRUE );
+                  v7_sendmsg(m->uin, m->text);
+                /*  icq_sendmsg( m->uin, m->text, TRUE ); */
 
 		g_free( m->data );
-		if( m->text != NULL )
-			g_free( m->text );
-
+		g_free( m->text );
 		g_free( m );
+
 		kontakt->tcp_msg_queue = g_slist_remove( kontakt->tcp_msg_queue, m );
 
 		return TRUE;
 	}
 
-        SendError = g_string_new("");
-        g_string_sprintf(SendError, "Unable to send a message directly to %s.", (gchar*)kontakt->nick);
-	label = gtk_label_new( (gchar*)SendError->str );
-        g_string_free(SendError, FALSE);
-	m->dialog = gnome_dialog_new( "GnomeICU: Question", "Retry", "Send Through Server", "Cancel", NULL );
- 	gtk_window_set_wmclass (GTK_WINDOW (m->dialog), "Question", "GnomeICU");
-	gtk_box_pack_start( GTK_BOX( GNOME_DIALOG( m->dialog )->vbox ), label,
-	                    FALSE, FALSE, GNOME_PAD );
+        error = g_strdup_printf( _("Unable to send a message directly to %s."), kontakt->nick );
+	m->dialog = gnome_message_box_new( error, GNOME_MESSAGE_BOX_QUESTION,
+					   _("Retry"), _("Send Through Server"),
+					   GNOME_STOCK_BUTTON_CANCEL, NULL );
+        g_free( error );
+	gtk_window_set_title( GTK_WINDOW( m->dialog ), _("GnomeICU: Question") );
+ 	gtk_window_set_wmclass( GTK_WINDOW( m->dialog ), "Question", "GnomeICU" );
+	gnome_dialog_set_default( GNOME_DIALOG( m->dialog ), 0 );
+	
+	gtk_signal_connect( GTK_OBJECT( m->dialog ), "clicked",
+	                    GTK_SIGNAL_FUNC( timeout_clicked ), m );
 
-	gnome_dialog_set_close( GNOME_DIALOG( m->dialog ), TRUE );
-
-	gnome_dialog_button_connect( GNOME_DIALOG(m->dialog), 0, timeout_continue, m );
-	gnome_dialog_button_connect( GNOME_DIALOG(m->dialog), 1, timeout_server, m );
-	gnome_dialog_button_connect( GNOME_DIALOG(m->dialog), 2, timeout_cancel, m );
-	gtk_widget_show( label );
 	gtk_widget_show( m->dialog );
 
 	return FALSE;
 }
 
-void timeout_continue( GtkWidget *widget, tcp_message *m )
+void timeout_clicked( GnomeDialog *dialog, gint button, tcp_message *m )
 {
 	GSList *contact;
 
@@ -2594,41 +2609,25 @@ void timeout_continue( GtkWidget *widget, tcp_message *m )
 	if( contact == NULL )
 		return;
 
-	m->timeout = gtk_timeout_add( 10000, (GtkFunction)TCPTimeout, m );
-	animate_on();
-}
+	switch (button) {
+		case 0: /* Retry */
+			m->timeout = gtk_timeout_add( 10000, (GtkFunction)TCPTimeout, m );
+			animate_on();
+			return;
 
-void timeout_server( GtkWidget *widget, tcp_message *m )
-{
-	GSList *contact;
+		case 1: /* Send through server */
+			if( m->text != NULL )
+                          v7_sendmsg(m->uin, m->text) ;
+                          /* icq_sendmsg( m->uin, m->text, TRUE ); */
 
-	/* We do this in case the user was removed before the message
-	   was sent.. */
-	contact = Find_User( m->uin );
-	if( contact == NULL )
-		return;
-
-	if( m->text != NULL )
-		icq_sendmsg( m->uin, m->text, TRUE );
-
-	g_free( m->data );
-	g_free( m->text );
-	g_free( m );
-
-	kontakt->tcp_msg_queue = g_slist_remove( kontakt->tcp_msg_queue, m );
-	kontakt->force_override = TRUE;
-}
-
-void timeout_cancel( GtkWidget *widget, tcp_message *m )
-{
-	GSList *contact;
-
-	/* We do this in case the user was removed before the message
-	   was sent.. */
-	contact = Find_User( m->uin );
-	if( contact == NULL )
-		return;
+			/* Next time force message through server */
+			kontakt->force_override = TRUE;
+			break;
 
+		case 2: /* Cancel */
+			break;
+	}
+	
 	g_free( m->data );
 	g_free( m->text );
 	g_free( m );
